<?php
/**
 * International-Tracked-and-Signed rate.
 *
 * @package WC_RoyalMail/Rate
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

use WooCommerce\RoyalMail\Services;
use WooCommerce\RoyalMail\Packaging;
use WooCommerce\RoyalMail\Shipping_Zones;

/**
 * RoyalMail_Online_International_Tracked_Signed class.
 *
 * Updated on 2025-01-11 as per https://www.royalmail.com/sites/royalmail.com/files/2025-01/online-price-guide-january-2025-v2.pdf.
 * See International Tracked & Signed page 12 - 14.
 */
class RoyalMail_Online_International_Tracked_Signed extends RoyalMail_Rate {
	const COMPENSATION_UP_TO_VALUE    = 250;
	const COMPENSATION_INCLUDED_VALUE = 50;

	/**
	 * Pricing bands
	 *
	 * @var array
	 */
	protected $bands;

	/**
	 * Additional rates for printed papers.
	 *
	 * @var array.
	 */
	protected $additional_rates;

	/**
	 * List of countries that support Tracked and Signed service.
	 *
	 * @see https://www.royalmail.com/business/system/files/Royal-Mail-International-Business-Services-List-March2017.pdf.
	 *
	 * @since 2.5.4
	 * @version 2.5.4
	 *
	 * @var array
	 */
	protected $supported_countries = array(
		'AX',
		'AD',
		'AR',
		'AT',
		'BB',
		'BY',
		'BE',
		'BZ',
		'BG',
		'KH',
		'CA',
		'KY',
		'CK',
		'HR',
		'CY',
		'CZ',
		'DK',
		'EC',
		'FO',
		'FI',
		'FR',
		'GE',
		'DE',
		'GI',
		'GR',
		'GL',
		'HK',
		'HU',
		'IS',
		'ID',
		'IE',
		'IT',
		'JP',
		'LV',
		'LB',
		'LI',
		'LT',
		'LU',
		'MY',
		'MT',
		'MD',
		'NL',
		'NZ',
		'PL',
		'PT',
		'RO',
		'RU',
		'SM',
		'RS',
		'SG',
		'SK',
		'SI',
		'KR',
		'ES',
		'SE',
		'CH',
		'TH',
		'TO',
		'TT',
		'TR',
		'UG',
		'AE',
		'US',
		'VA',
	);

	/**
	 * Setup RoyalMail_Online_International_Tracked_Signed rates.
	 */
	public function setup() {
		$this->bands            = $this->initialize_bands();
		$this->additional_rates = $this->initialize_additional_rates();
	}

	/**
	 * Initializes pricing bands for package sizes and destinations for the year 2024.
	 * The pricing structure includes rates for countries categorized by different zones:
	 * Ireland, France, Germany, Spain, Italy, Canada, China, Japan, Australia, New Zealand, Hong Kong, Europe 1, Europe 2, Europe 3, World Zone 1, World Zone 2, World Zone 3 (previously Zone 1).
	 *
	 * @return array The array of pricing bands.
	 */
	private function initialize_bands() {
		return array(
			'2024' => array(
				Packaging::LETTER         => array(
					100 => array( 815, 815, 815, 815, 815, 815, 815, 815, 815, 815, 815, 815, 815, 815, 815, 815, 815 ),
				),
				Packaging::LARGE_LETTER   => array(
					100 => array( 965, 965, 965, 965, 965, 1065, 1065, 1065, 1080, 1080, 1065, 965, 965, 965, 1065, 1080, 1075 ),
					250 => array( 1020, 1020, 1020, 1020, 1020, 1160, 1160, 1160, 1280, 1280, 1160, 1020, 1020, 1020, 1160, 1280, 1180 ),
					500 => array( 1100, 1100, 1100, 1100, 1100, 1360, 1360, 1360, 1560, 1560, 1360, 1100, 1100, 1100, 1360, 1560, 1395 ),
					750 => array( 1145, 1145, 1145, 1145, 1145, 1565, 1565, 1565, 1865, 1865, 1565, 1145, 1145, 1145, 1565, 1865, 1615 ),
				),
				Packaging::PACKET         => array(
					100  => array( 1015, 1015, 1015, 1015, 1015, 1540, 1540, 1540, 1330, 1330, 1540, 1015, 970, 1095, 1540, 1330, 1280 ),
					250  => array( 1015, 1015, 1015, 1015, 1015, 1575, 1575, 1575, 1365, 1365, 1575, 1015, 970, 1095, 1575, 1365, 1440 ),
					500  => array( 1135, 1135, 1135, 1135, 1135, 1665, 1665, 1665, 1815, 1815, 1665, 1135, 1125, 1250, 1665, 1815, 1880 ),
					750  => array( 1240, 1240, 1240, 1240, 1240, 1905, 1905, 1905, 2095, 2095, 1905, 1240, 1225, 1365, 1905, 2095, 1880 ),
					1000 => array( 1335, 1335, 1335, 1335, 1335, 2170, 2170, 2170, 2410, 2410, 2170, 1335, 1315, 1490, 2170, 2410, 2045 ),
					1250 => array( 1390, 1390, 1390, 1390, 1390, 2370, 2370, 2370, 2705, 2705, 2370, 1390, 1345, 1645, 2370, 2705, 2390 ),
					1500 => array( 1400, 1400, 1400, 1400, 1400, 2550, 2550, 2550, 3005, 3005, 2550, 1400, 1370, 1755, 2550, 3005, 2665 ),
					2000 => array( 1415, 1415, 1415, 1415, 1415, 2655, 2655, 2655, 3210, 3210, 2655, 1415, 1415, 2135, 2655, 3210, 2715 ),
				),
				Packaging::PRINTED_PAPERS => array(
					100  => array( 1015, 1015, 1015, 1015, 1015, 1540, 1540, 1540, 1330, 1330, 1540, 1015, 970, 1095, 1540, 1330, 1280 ),
					250  => array( 1015, 1015, 1015, 1015, 1015, 1575, 1575, 1575, 1365, 1365, 1575, 1015, 970, 1095, 1575, 1365, 1440 ),
					500  => array( 1135, 1135, 1135, 1135, 1135, 1665, 1665, 1665, 1815, 1815, 1665, 1135, 1125, 1250, 1665, 1815, 1880 ),
					750  => array( 1240, 1240, 1240, 1240, 1240, 1905, 1905, 1905, 2095, 2095, 1905, 1240, 1225, 1365, 1905, 2095, 1880 ),
					1000 => array( 1335, 1335, 1335, 1335, 1335, 2170, 2170, 2170, 2410, 2410, 2170, 1335, 1315, 1490, 2170, 2410, 2045 ),
					1250 => array( 1390, 1390, 1390, 1390, 1390, 2370, 2370, 2370, 2705, 2705, 2370, 1390, 1345, 1645, 2370, 2705, 2390 ),
					1500 => array( 1400, 1400, 1400, 1400, 1400, 2550, 2550, 2550, 3005, 3005, 2550, 1400, 1370, 1755, 2550, 3005, 2665 ),
					2000 => array( 1415, 1415, 1415, 1415, 1415, 2655, 2655, 2655, 3210, 3210, 2655, 1415, 1415, 2135, 2655, 3210, 2715 ),
				),
				Packaging::MEDIUM_PARCEL  => array(
					100   => array( 1070, 1120, 1020, 1275, 1275, 2250, 2250, 2535, 2695, 2695, 2250, 1400, 1275, 1570, 2535, 2695, 2345 ),
					250   => array( 1070, 1120, 1020, 1275, 1275, 2250, 2250, 2535, 2695, 2695, 2250, 1400, 1275, 1570, 2535, 2695, 2345 ),
					500   => array( 1070, 1120, 1020, 1275, 1275, 2250, 2250, 2535, 2695, 2695, 2250, 1400, 1275, 1570, 2535, 2695, 2345 ),
					750   => array( 1070, 1120, 1020, 1275, 1275, 2250, 2250, 2535, 2695, 2695, 2250, 1400, 1275, 1570, 2535, 2695, 2345 ),
					1000  => array( 1070, 1120, 1020, 1275, 1275, 2250, 2250, 2535, 2695, 2695, 2250, 1400, 1275, 1570, 2535, 2695, 2345 ),
					1250  => array( 1070, 1170, 1020, 1580, 1480, 2340, 2750, 2825, 3440, 3440, 2750, 1460, 1580, 2070, 2825, 3440, 3215 ),
					1500  => array( 1070, 1170, 1020, 1580, 1480, 2340, 2750, 2825, 3440, 3440, 2750, 1460, 1580, 2070, 2825, 3440, 3215 ),
					2000  => array( 1070, 1170, 1020, 1580, 1480, 2340, 2750, 2825, 3440, 3440, 2750, 1460, 1580, 2070, 2825, 3440, 3215 ),
					3000  => array( 1090, 1260, 1180, 1830, 1830, 2745, 3200, 3200, 3810, 3810, 3200, 1665, 1830, 2575, 3225, 3810, 3340 ),
					4000  => array( 1160, 1300, 1270, 3295, 3295, 3070, 3815, 3815, 4230, 4230, 3815, 1715, 3295, 3255, 3865, 4230, 3920 ),
					5000  => array( 1290, 1385, 1395, 3595, 3595, 3395, 4745, 4745, 4670, 4870, 4745, 1770, 3595, 3835, 4845, 5080, 4500 ),
					7500  => array( 1480, 1440, 1765, 5340, 5340, 4180, 6405, 6405, 6270, 6670, 6405, 2315, 5340, 5360, 6525, 6920, 6100 ),
					10000 => array( 1605, 1675, 2130, 7310, 7310, 4910, 9780, 9780, 8600, 9100, 9780, 2815, 7310, 7320, 9780, 9600, 7600 ),
					15000 => array( 1815, 1995, 2865, 10160, 10160, 5240, 12350, 12350, 12100, 12100, 12350, 3810, 10160, 9915, 12550, 13500, 11070 ),
					20000 => array( 1919, 2560, 3600, 15710, 15710, 5440, 17000, 17000, 16700, 16700, 17000, 4755, 15710, 12215, 17300, 18700, 13300 ),
				),
			),
		);
	}

	/**
	 * Initialize the additional rates for printed papers.
	 *
	 * @return array The array of additional rates.
	 */
	private function initialize_additional_rates() {
		return array(
			'2024' => array(
				Packaging::PRINTED_PAPERS => array( 130, 130, 130, 130, 130, 180, 180, 180, 225, 225, 180, 130, 130, 130, 180, 225, 250 ),
			),
		);
	}

	/**
	 * Fixed compensation
	 *
	 * @var string
	 */
	private $compensation = '250';

	/**
	 * Slug of the rate (e.g. 'special-delivery-1pm').
	 *
	 * @return string
	 */
	public function get_rate_slug() {
		return Services::INTERNATIONAL_TRACKED_SIGNED;
	}

	/**
	 * Get quotes for this rate.
	 *
	 * @since 2.5.4
	 * @version 2.5.4
	 *
	 * @param  array  $items to be shipped.
	 * @param  string $packing_method the method selected.
	 * @param  string $destination Address to ship to.
	 * @param  array  $boxes User-defined boxes.
	 * @param  int    $instance_id Instance ID.
	 *
	 * @return array{ 'international-tracked-signed': float }|false|null
	 */
	public function get_quotes( $items, $packing_method, $destination, $boxes = array(), $instance_id = '' ) {
		if ( ! in_array( $destination, $this->supported_countries, true ) ) {
			return array();
		}

		$this->apply_additional_rates();

		$class_quote = false;

		if ( ! empty( $boxes ) ) {
			$this->boxes = array();

			foreach ( $boxes as $key => $box ) {
				$this->boxes[ $key ] = array(
					'length'     => $box['inner_length'],
					'width'      => $box['inner_width'],
					'height'     => $box['inner_height'],
					'box_weight' => $box['box_weight'],
					'weight'     => 2000,
				);
			}
		} else {
			$this->boxes = $this->international_default_box;
		}

		$individual_zones = array( 'IE', 'FR', 'DE', 'ES', 'IT', 'CA', 'CN', 'JP', 'AU', 'NZ', 'HK' );
		$zone             = in_array( $destination, $individual_zones, true ) ? $destination : $this->get_zone( $destination );

		/**
		 * Allow third party to enable/disable printed papers rate.
		 *
		 * @param boolean $rate_enabled Flag for enabling/disabling the rate.
		 * @param int $instance_id Instance ID.
		 * @param string $rate_slug Name of the rate.
		 * @param array $destination Destination.
		 * @param string $packing_method Packing method.
		 *
		 * @since 3.0.0
		 */
		$printed_paper_packages  = apply_filters( 'woocommerce_shipping_royal_mail_printed_papers_enabled', true, $instance_id, 'tracked-signed', $destination, $packing_method ) ? $this->get_printed_papers_packages( $items, $destination, $packing_method ) : array();
		$regular_packages        = $this->get_packages( $items, $packing_method );
		$packages                = array_merge( $regular_packages, $printed_paper_packages );
		$options                 = $this->get_instance_options( $instance_id );
		$ignore_max_compensation = ( ! empty( $options['compensation_optional'] ) && 'yes' === $options['compensation_optional'] );
		$additional_compensation = ( ! empty( $options['enable_addit_compensation'] ) && 'yes' === $options['enable_addit_compensation'] );
		$max_compensation        = ( true === $additional_compensation ) ? self::COMPENSATION_UP_TO_VALUE : self::COMPENSATION_INCLUDED_VALUE;

		if ( $packages ) {
			foreach ( $packages as $package ) {

				$this->validate_package( $package );

				if ( in_array( $package->id, array( Packaging::PACKET, Packaging::PRINTED_PAPERS ), true ) && 900 < ( $package->length + $package->width + $package->height ) ) {
					return false; // Exceeding parcels requirement, unpacked.
				}

				if ( $package->value > $max_compensation && ! $ignore_max_compensation ) {
					return false;
				}

				if ( ! $this->get_rate_bands( $package->id ) ) {
					return false; // Unpacked item.
				}

				$bands   = $this->get_rate_bands( $package->id );
				$quote   = 0;
				$matched = false;

				foreach ( $bands as $band => $value ) {
					if ( $package->weight <= $band ) {
						switch ( $zone ) {
							case 'IE':
								$quote += $value[0];
								break;
							case 'FR':
								$quote += $value[1];
								break;
							case 'DE':
								$quote += $value[2];
								break;
							case 'ES':
								$quote += $value[3];
								break;
							case 'IT':
								$quote += $value[4];
								break;
							case 'CA':
								$quote += $value[5];
								break;
							case 'CN':
								$quote += $value[6];
								break;
							case 'JP':
								$quote += $value[7];
								break;
							case 'AU':
								$quote += $value[8];
								break;
							case 'NZ':
								$quote += $value[9];
								break;
							case 'HK':
								$quote += $value[10];
								break;
							case Shipping_Zones::ZONE_EUR_1:
								$quote += $value[11];
								break;
							case Shipping_Zones::ZONE_EUR_2:
								$quote += $value[12];
								break;
							case Shipping_Zones::ZONE_EUR_3:
							case Shipping_Zones::ZONE_EU:
								$quote += $value[13];
								break;
							case Shipping_Zones::ZONE_1:
								$quote += $value[14];
								break;
							case Shipping_Zones::ZONE_2:
								$quote += $value[15];
								break;
							case Shipping_Zones::ZONE_3:
								// Fallback to zone 1 for older prices.
								$quote += isset( $value[16] ) ? $value[16] : $value[14];
								break;
						}
						$matched = true;
						break;
					}
				}

				if ( ! $matched ) {
					return;
				}

				$class_quote += $quote;

				if ( $package->value > self::COMPENSATION_INCLUDED_VALUE && $additional_compensation ) {
					$class_quote += $this->compensation;
				}
			}
		}

		// Return pounds.
		$quotes                           = array();
		$quotes[ $this->get_rate_slug() ] = $class_quote / 100;

		return $quotes;
	}
}
