<?php
/**
 * UK Signed For Second class signed rate.
 *
 * @package WC_RoyalMail/Rate
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

use WooCommerce\RoyalMail\Services;
use WooCommerce\RoyalMail\Packaging;

/**
 * RoyalMail_Online_Second_Class_Signed class.
 *
 * Updated on 2025-01-11 as per https://www.royalmail.com/sites/royalmail.com/files/2025-01/online-price-guide-january-2025-v2.pdf.
 * See UK Signed For page 6.
 */
class RoyalMail_Online_Second_Class_Signed extends RoyalMail_Online_Second_Class {

	const COMPENSATION_UP_TO_VALUE = 50;

	/**
	 * Cost for signed for delivery.
	 *
	 * @var array
	 */
	protected $signed_for_cost = array(
		'2024' => 170,
	);

	/**
	 * Cost for signed for delivery of a package.
	 *
	 * @var array
	 */
	protected $signed_for_package_cost = array(
		'2024' => 140,
	);

	/**
	 * Slug of the rate (e.g. 'special-delivery-1pm').
	 *
	 * @return string
	 */
	public function get_rate_slug() {
		return Services::SECOND_CLASS_SIGNED;
	}

	/**
	 * Get quotes for this rate.
	 *
	 * @param array  $items to be shipped.
	 * @param string $packing_method selected.
	 * @param string $destination address.
	 * @param array  $boxes Boxes used.
	 * @param int    $instance_id Instance ID.
	 *
	 * @return array{ 'second-class-signed': float }|false|null
	 */
	public function get_quotes( $items, $packing_method, $destination, $boxes = array(), $instance_id = '' ) {
		$class_quote             = false;
		$packages                = $this->get_packages( $items, $packing_method );
		$options                 = $this->get_instance_options( $instance_id );
		$ignore_max_compensation = ( ! empty( $options['compensation_optional'] ) && 'yes' === $options['compensation_optional'] );

		if ( $packages ) {
			foreach ( $packages as $package ) {
				if ( $package->value > self::COMPENSATION_UP_TO_VALUE && ! $ignore_max_compensation ) {
					return false; // Max. compensation is 50.
				}

				$quote = 0;

				if ( ! $this->get_rate_bands( $package->id ) ) {
					return false; // Unpacked item.
				}


				$bands = $this->get_rate_bands( $package->id );

				$matched = false;

				foreach ( $bands as $band => $value ) {
					if ( is_numeric( $band ) && $package->weight <= $band ) {
						$quote  += $value;
						$matched = true;
						break;
					}
				}

				if ( ! $matched ) {
					return;
				}

				if ( Packaging::LETTER === $package->id || Packaging::LARGE_LETTER === $package->id ) {
					$class_quote += $quote + $this->get_signed_cost( 'signed_for_cost' );
				} else {
					$class_quote += $quote + $this->get_signed_cost( 'signed_for_package_cost' );
				}
			}
		}

		// Return pounds.
		$quotes                           = array();
		$quotes[ $this->get_rate_slug() ] = $class_quote / 100;

		return $quotes;
	}
}
